@@ -16,8 +16,24 @@ testedwith = '2.4'
 buglink = 'https://www.logilab.org/project/hgview'
 
 import os
+import os.path as osp
+import stat
+import imp
 from optparse import Values
-from mercurial import error
+from mercurial import error, demandimport
+
+execpath = osp.abspath(__file__)
+#   resolve symbolic links
+statinfo = os.lstat(execpath)
+if stat.S_ISLNK(statinfo.st_mode):
+    execpath = join(osp.dirname(execpath), os.readlink(execpath))
+    execpath = osp.abspath(execpath)
+# if standalone, import manually
+setuppath = osp.join(osp.dirname(osp.dirname(execpath)), 'setup.py')
+if osp.exists(setuppath): # standalone if setup.py found in src dir
+    hgviewlibpath = osp.join(osp.dirname(osp.dirname(execpath)), 'hgviewlib')
+    hgviewlibpath = osp.abspath(hgviewlibpath)
+    imp.load_package('hgviewlib', hgviewlibpath)
 
 # every command must take a ui and and repo as arguments.
 # opts is a dict where you can find other command line flags
@@ -66,9 +82,12 @@ start hgview log viewer
     options = Values(opts)
     start(repo, options, pats, fnerror)
 
-import hgviewlib.hgviewhelp as hghelp
+import hgviewlib
 
-start_hgview.__doc__ = hghelp.long_help_msg
+# note: ``import hgviewlib.hgviewhelp`` is incompatible with standalone
+#       because of the lazy import
+from hgviewlib.hgviewhelp import long_help_msg
+start_hgview.__doc__ = long_help_msg
 
 cmdtable = {
     "^hgview|hgv|qv": (start_hgview,
