@@ -19,9 +19,10 @@ revision grapher.
 """
 
 import os
+import re
 from cStringIO import StringIO
 import difflib
-from itertools import chain, count
+from itertools import chain, count, imap
 from time import strftime, localtime
 from functools import partial
 
@@ -31,10 +32,15 @@ from mercurial import patch, util, match, error, hg
 import hgviewlib.hgpatches # force apply patches to mercurial
 from hgviewlib.hgpatches import mqsupport, phases, hiddenrevs
 
-from hgviewlib.util import tounicode, isbfile, first_known_precursors, build_repo
+from hgviewlib.util import (tounicode, isbfile, first_known_precursors,\
+                            build_repo, allbranches)
 from hgviewlib.config import HgConfig
 
 DATE_FMT = '%F %R'
+# match the end of the diff header, assuming that the following line
+# looks like "@@ -25,6 +23,5 @@"
+DIFFHEADERMATCHER = re.compile('^@@.+@@$', re.MULTILINE)
+
 
 def diff(repo, ctx1, ctx2=None, files=None):
     """
@@ -67,13 +73,7 @@ def diff(repo, ctx1, ctx2=None, files=None):
     except:
         diffdata = '\n'.join(patch.diff(repo, ctx2.node(), ctx1.node(),
                                         match=matchfn, opts=diffopts))
-    # XXX how to deal diff encodings?
-    try:
-        diffdata = unicode(diffdata, "utf-8")
-    except UnicodeError:
-        # XXX use a default encoding from config?
-        diffdata = unicode(diffdata, "iso-8859-15", 'ignore')
-    return diffdata
+    return tounicode(diffdata)
 
 
 def __get_parents(repo, rev, branch=None):
@@ -98,17 +98,17 @@ def getlog(model, ctx, gnode):
         if msg:
             msg = msg.splitlines()[0]
     else:
-        msg = "WORKING DIRECTORY (locally modified)"
+        msg = u"WORKING DIRECTORY (locally modified)"
     return msg
 
 def gettags(model, ctx, gnode):
     if ctx.rev() is None:
-        return ""
+        return u""
     mqtags = ['qbase', 'qtip', 'qparent']
     tags = ctx.tags()
     if model.hide_mq_tags:
         tags = [t for t in tags if t not in mqtags]
-    return ",".join(tags)
+    return u",".join(imap(tounicode, tags))
 
 def getdate(model, ctx, gnode):
     if not ctx.date():
@@ -124,43 +124,93 @@ def ismerge(ctx):
         return len(ctx.parents()) == 2 and ctx.parents()[1]
     return False
 
-def _graph_iterator(repo, start_rev, stop_rev, reorder=False):
-    """Iter through revisions from start_rev to stop_rev (included)
-    Handle Working directory as None.
-    """
-    # check parameters
-    assert start_rev is None or start_rev >= stop_rev
-    assert start_rev < len(repo)
-    if start_rev is None:
-        yield start_rev
-        start_rev = len(repo.changelog) -1
+def _get_phaserevs(repo):
+    """return phase information for all rev in repo
 
-    target_revs = xrange(start_rev, stop_rev-1, -1)
+    this function is useful to stay compatible with multiple Mercurial version
+
+    if the version does not support phases the function return None"""
     phaserevs = None
     if getattr(repo, '_phasecache', None) is not None:
         phaserevs = repo._phasecache._phaserevs
     elif getattr(repo, '_phaserev', None) is not None:
         phaserevs = repo._phaserev
-    if not reorder or phaserevs is None:
-        # old hg
-        for curr_rev in target_revs:
-            yield curr_rev
+    return phaserevs
+
+def _dirty_wc(repo):
+    """return true if the working directory has changes"""
+    return repo.status() != ([],)*7  # dirty working dir
+
+def _rev_order(phaserevs, rev):
+    """return a sort key for changeset reordering (<freshness>, rev)
+
+    Freshness is:
+    :0: public revision
+    :1: mutable revision
+    :42: working directory
+    """
+    if rev is None:
+        return (42, rev)
+    return (phases.public != phaserevs[rev], rev)
+
+def _build_filter(start_rev, branch, follow, closed):
+    """process filter rules and returns maching revisions as list"""
+    # selected range
+    revset_args = []
+    if (start_rev is None):
+        revset = 'all()'
+    elif follow:
+        revset = '::(%s)' % start_rev
     else:
-        _cmp = lambda a, b: cmp(phases.public == phaserevs[a],
-                                phases.public == phaserevs[b])
-        for r in sorted(target_revs, cmp=_cmp):
-            yield r
+        revset = ':(%s)' % start_rev
 
+    # branch restriction
+    if branch:
+        # we add both changeset that belong to the branch and other merged.
+        revset = '(%s and (branch(%%s) or parents(branch(%%s))))' % (revset)
+        revset_args.extend([branch, branch])
 
+    # closed branch restriction
+    if not closed:
+        revset = '(%s and ::(head() - closed()))' % revset
+    return revset, revset_args
 
-def revision_grapher(repo, start_rev=None, stop_rev=0, branch=None, follow=False,
+def _revset_revs(repo, revset, revset_args=(), view='visible'):
+    """Use the fastest available method to get revs from a revset
+
+    The view argument is used to control the available revision (see `repoview`
+    in Mercurila core). We currently support "visible" and None as we have to
+    implement backward compat manually.
+
+    The revs are returned in ordered."""
+
+    revset = 'sort(%s)' % revset
+
+    assert view in ('visible', None)
+    if getattr(repo, 'filtered', None) is not None:
+        # Mercurial 2.5 and above. repoview available, relying on it.
+        if view is None:
+            repo = repo.unfiltered()
+        else:
+            repo = repo.filtered(view)
+        return repo.revs(revset, *revset_args)
+    else:
+        if getattr(repo, 'revs', None) is None: # pre 2.1 hg
+            revs = [c.rev() for c in repo.set(revset, *revset_args)]
+        else:
+            revs = repo.revs(revset, *revset_args)
+        excluded = () if view is None else hiddenrevs(repo)
+        if excluded:
+            revs = [r for r in revs if r not in excluded]
+        return revs
+
+def revision_grapher(repo, start_rev=None, branch=None, follow=False,
                      show_hidden=False, reorder=False, closed=False,
                      show_obsolete=False):
     """incremental revision grapher
 
     This generator function walks through the revision history from
-    revision start_rev to revision stop_rev (which must be less than
-    or equal to start_rev) and for each revision emits tuples with the
+    revision start_rev for each revision emits tuples with the
     following elements:
 
       - current revision
@@ -170,8 +220,6 @@ def revision_grapher(repo, start_rev=None, stop_rev=0, branch=None, follow=False
         the current row and the next row
       - parent revisions of current revision
 
-    If start_rev is -1, shown unapplied mq patches (if available)
-
     If start_rev is None, started from working directory node
 
     If follow is True, only generated the subtree from the start_rev head.
@@ -182,16 +230,35 @@ def revision_grapher(repo, start_rev=None, stop_rev=0, branch=None, follow=False
     # follow is disabled when start_rev is None (== not revision specified)
     follow = start_rev is not None and follow # disable follow is start_rev is None
 
+    # special handling of mq patches
     if show_hidden and start_rev == None and hasattr(repo, 'mq'):
         series = list(reversed(repo.mq.series))
         for patchname in series:
             if not repo.mq.isapplied(patchname):
                 yield (patchname, 0, 0, [(0, 0 ,0, False)], [])
 
-    # No uncommitted change
-    if start_rev is None and repo.status() == ([],)*7:
-        start_rev = len(repo.changelog) -1
-    assert start_rev is None or start_rev >= stop_rev
+    # computed hidden revs
+    excluded = () if show_hidden else hiddenrevs(repo)
+
+    revset, args = _build_filter(start_rev, branch, follow, closed)
+    view = None if show_hidden else 'visible'
+    included = _revset_revs(repo, revset, args, view=view)
+    if getattr(included, 'append', None) is None:
+        # _revset_revs returned a smartset (new in Mercurial 3.0)
+        # translate it back to list for now
+        included = list(included)
+
+    # working directory
+    if start_rev is None and _dirty_wc(repo):
+        included.append(None)
+
+    # perform reordering
+    phaserevs = _get_phaserevs(repo)
+    if not (show_hidden or phaserevs is None or not reorder):
+        keyfunc = partial(_rev_order, phaserevs)
+        included.sort(key=keyfunc)
+
+    included.reverse()
 
     # all known revs for this line. This is used to compute column index
     # it's combined with next_revs to compute how we must draw lines
@@ -201,19 +268,9 @@ def revision_grapher(repo, start_rev=None, stop_rev=0, branch=None, follow=False
                     #            False for weak one (obsolete)
     rev_color = {}
     free_color = count(0)
-    excluded = () if show_hidden else hiddenrevs(repo)
-    closedbranches = [tag for tag, node in repo.branchtags().items()
-                      if repo.lookup(node) not in repo.branchheads(tag, closed=False)]
-    for curr_rev in _graph_iterator(repo, start_rev, stop_rev, not show_hidden and reorder):
+    for curr_rev in included:
         # Compute revs and next_revs.
-        if curr_rev in excluded:
-            continue
         if curr_rev not in revs: # rev not ancestor of already processed node
-            # shall we ignore this new heads ?
-            if (branch and repo[curr_rev].branch() != branch) or \
-               (follow and curr_rev != start_rev) or \
-               (not closed and repo[curr_rev].branch() in closedbranches):
-                continue
             # we add this new head to know revision
             revs.append(curr_rev)
             levels.append(True)
@@ -481,12 +538,14 @@ class Graph(object):
     def filename(self, rev):
         return self.nodesdict[rev].extra[0]
 
-    def filedata(self, filename, rev, mode='diff', flag=None):
+    def filedata(self, filename, rev, mode='diff', flag=None, maxfilesize=None):
         """XXX written under dubious encoding assumptions
 
         The modification flag is computed using *fileflag* if ``flag`` is None.
         """
         # XXX This really begins to be a dirty mess...
+        if maxfilesize is None:
+            maxfilesize = self.maxfilesize
         data = ""
         if flag is None:
             flag = self.fileflag(filename, rev)
@@ -498,30 +557,30 @@ class Graph(object):
         except (LookupError, OSError):
             fctx = None # may happen for renamed/removed files or mq patch ?
         if isbfile(filename):
-            data = "[bfile]\n"
+            data = u"[bfile]\n"
             if fctx:
                 data = fctx.data()
-                data += "footprint: %s\n" % data
+                data += u"footprint: %s\n" % tounicode(data)
             return "+", data
         if flag not in ('-', '?'):
             if fctx is None:# or fctx.node() is None:
                 return '', None
-            if self.maxfilesize >= 0 and filesize > self.maxfilesize:
+            if maxfilesize >= 0 and filesize > maxfilesize:
                 try:
                     div = int(filesize).bit_length() // 10
                     sym = ('', 'K', 'M', 'G', 'T', 'E')[div] # more, really ???
                     val = int(filesize / (2 ** (div * 10)))
-                except AttributeError: # py<2.7
+                except AttributeError: # py < 2.7
                     val = filesize
                     sym = ''
-                data = "File too big ! (~%i%so)" % (val, sym)
-                return flag, data
+                data = u"~%i%so" % (val, sym)
+                return 'file too big', data
             if flag == "+" or mode == 'file':
                 flag = '+'
                 # return the whole file
                 data = fctx.data()
                 if util.binary(data):
-                    data = "binary file"
+                    data = u"binary file"
                 else: # tries to convert to unicode
                     data = tounicode(data)
             elif flag == "=" or isinstance(mode, int):
@@ -531,10 +590,14 @@ class Graph(object):
                 else:
                     parentctx = self.repo[self._fileparent(fctx)]
                 data = diff(self.repo, ctx, parentctx, files=[filename])
-                # we assume that \n@@ marks the end of the diff header
-                data = ''.join(data.partition(os.linesep + '@@')[1:]).lstrip()
+                match = DIFFHEADERMATCHER.search(data)
+                if match is not None:
+                    datastart = match.start()
+                else:
+                    datastart = 0
+                data = data[datastart:]
             elif flag == '':
-                data = ''
+                data = u''
             else: # file renamed
                 oldname, node = flag
                 newdata = fctx.data().splitlines()
@@ -610,7 +673,8 @@ class HgRepoListWalker(object):
         self._user_colors = {}
         self._branch_colors = {}
         # precompute named branch color for stable value.
-        for branch_name in chain(['default', 'stable'], sorted(repo.branchtags().keys())):
+        for branch_name in chain(['default', 'stable'],
+                                 sorted(allbranches(repo, True))):
             self.namedbranch_color(branch_name)
         grapher = revision_grapher(self.repo, start_rev=fromhead,
                                    follow=follow, branch=branch,
@@ -676,16 +740,8 @@ class HgRepoListWalker(object):
         cols = getattr(cfg, self._getcolumns)()
         if cols is not None:
             validcols = [col for col in cols if col in self._allcolumns]
-            if len(validcols) != len(cols):
-                wrongcols = [col for col in cols if col not in self._allcolumns]
-                #XXX
-                #print "WARNING! %s are not valid column names. Check your configuration." % ','.join(wrongcols)
-                #print "         reverting to default columns configuration"
-            elif 'Log' not in validcols or 'ID' not in validcols:
-                pass
-                #print "WARNING! 'Log' and 'ID' are mandatory. Check your configuration."
-                #print "         reverting to default columns configuration"
-            else:
+            if len(validcols) == len(cols) and \
+               set(['Log', 'ID']).issubset(set(validcols)) :
                 self._columns = tuple(validcols)
 
     @staticmethod
