@@ -25,7 +25,7 @@ import difflib
 from mercurial import ui, hg, util
 
 from PyQt4 import QtGui, QtCore, Qsci
-from PyQt4.QtCore import Qt
+from PyQt4.QtCore import Qt, pyqtSignal
 
 from hgviewlib.application import (FileViewer as _FileViewer,
                                    FileDiffViewer as _FileDiffViewer)
@@ -38,10 +38,6 @@ from hgviewlib.qt4.blockmatcher import BlockList, BlockMatch
 from hgviewlib.qt4.lexers import get_lexer
 from hgviewlib.qt4.quickbar import FindInGraphlogQuickBar
 
-connect = QtCore.QObject.connect
-disconnect = QtCore.QObject.disconnect
-SIGNAL = QtCore.SIGNAL
-nullvariant = QtCore.QVariant()
 
 sides = ('left', 'right')
 otherside = {'left': 'right', 'right': 'left'}
@@ -68,8 +64,7 @@ class AbstractFileDialog(QtGui.QMainWindow, HgDialogMixin):
     def setRepoViewer(self, repoviewer=None):
         self.repoviewer = repoviewer
         if repoviewer:
-            connect(repoviewer, SIGNAL('finished(int)'),
-                    lambda x: self.setRepoViewer())
+            repoviewer.destroyed.connect(lambda x: self.setRepoViewer())
 
     def reload(self):
         self.repo = util.build_repo(self.repo.ui, self.repo.root)
@@ -90,8 +85,7 @@ class AbstractFileDialog(QtGui.QMainWindow, HgDialogMixin):
         self.lexer = lexer
 
     def modelFilled(self):
-        disconnect(self.filerevmodel, SIGNAL('filled'),
-                   self.modelFilled)
+        self.filerevmodel.filled.disconnect(self.modelFilled)
         if isinstance(self._show_rev, int):
             index = self.filerevmodel.indexFromRev(self._show_rev)
             self._show_rev = None
@@ -120,16 +114,17 @@ class FileViewer(AbstractFileDialog, _FileViewer):
 
     def setupViews(self):
         self.textView.setFont(self._font)
-        connect(self.textView, SIGNAL('showMessage'),
-                self.statusBar().showMessage)
+        self.setWindowTitle('hgview filelog: %s' % os.path.abspath(self.filename))
+        self.textView.message_logged.connect(self.statusBar().showMessage)
 
     def setupToolbars(self):
         self.find_toolbar = FindInGraphlogQuickBar(self)
         self.find_toolbar.attachFileView(self.textView)
-        connect(self.find_toolbar, SIGNAL('revisionSelected'),
-                self.tableView_revisions.goto)
-        connect(self.find_toolbar, SIGNAL('showMessage'),
-                self.statusBar().showMessage)
+        self.find_toolbar.revision_selected.connect(
+            self.tableView_revisions.goto)
+        self.find_toolbar.revision_selected[int].connect(
+            self.tableView_revisions.goto)
+        self.find_toolbar.message_logged.connect(self.statusBar().showMessage)
         self.attachQuickBar(self.find_toolbar)
 
         self.toolBar_edit.addSeparator()
@@ -146,17 +141,18 @@ class FileViewer(AbstractFileDialog, _FileViewer):
     def setupModels(self):
         self.filerevmodel = FileRevModel(self.repo)
         self.tableView_revisions.setModel(self.filerevmodel)
-        connect(self.tableView_revisions,
-                SIGNAL('revisionSelected'),
-                self.revisionSelected)
-        connect(self.tableView_revisions,
-                SIGNAL('revisionActivated'),
+        self.tableView_revisions.revision_selected.connect(
+            self.revisionSelected)
+        self.tableView_revisions.revision_selected[int].connect(
+            self.revisionSelected)
+        self.tableView_revisions.revision_activated.connect(
                 self.revisionActivated)
-        connect(self.filerevmodel, SIGNAL('showMessage'),
-                self.statusBar().showMessage,
-                Qt.QueuedConnection)
-        connect(self.filerevmodel, QtCore.SIGNAL('filled'),
-                self.modelFilled)
+        self.tableView_revisions.revision_activated[int].connect(
+                self.revisionActivated)
+        self.filerevmodel.message_logged.connect(
+            self.statusBar().showMessage,
+            Qt.QueuedConnection)
+        self.filerevmodel.filled.connect(self.modelFilled)
         self.textView.setMode('file')
         self.textView.setModel(self.filerevmodel)
         self.find_toolbar.setModel(self.filerevmodel)
@@ -165,31 +161,26 @@ class FileViewer(AbstractFileDialog, _FileViewer):
         self.filerevmodel.setFilename(self.filename)
 
     def createActions(self):
-        connect(self.actionClose, SIGNAL('triggered()'),
-                self.close)
-        connect(self.actionReload, SIGNAL('triggered()'),
-                self.reload)
+        self.actionClose.triggered.connect(self.close)
+        self.addAction(self.actionClose)
+        self.actionReload.triggered.connect(self.reload)
         self.actionClose.setIcon(geticon('quit'))
         self.actionReload.setIcon(geticon('reload'))
 
         self.actionDiffMode = QtGui.QAction('Diff mode', self)
         self.actionDiffMode.setCheckable(True)
-        connect(self.actionDiffMode, SIGNAL('toggled(bool)'),
-                self.setMode)
+        self.actionDiffMode.toggled[bool].connect(self.setMode)
 
         self.actionAnnMode = QtGui.QAction('Annotate', self)
         self.actionAnnMode.setCheckable(True)
-        connect(self.actionAnnMode, SIGNAL('toggled(bool)'),
-                self.textView.setAnnotate)
+        self.actionAnnMode.toggled[bool].connect(self.textView.setAnnotate)
 
         self.actionNextDiff = QtGui.QAction(geticon('down'), 'Next diff', self)
         self.actionNextDiff.setShortcut('Alt+Down')
         self.actionPrevDiff = QtGui.QAction(geticon('up'), 'Previous diff', self)
         self.actionPrevDiff.setShortcut('Alt+Up')
-        connect(self.actionNextDiff, SIGNAL('triggered()'),
-                self.nextDiff)
-        connect(self.actionPrevDiff, SIGNAL('triggered()'),
-                self.prevDiff)
+        self.actionNextDiff.triggered.connect(self.nextDiff)
+        self.actionPrevDiff.triggered.connect(self.prevDiff)
 
     def revisionSelected(self, rev):
         pos = self.textView.verticalScrollBar().value()
@@ -198,7 +189,7 @@ class FileViewer(AbstractFileDialog, _FileViewer):
         self.textView.displayFile(self.filerevmodel.graph.filename(rev))
         self.textView.verticalScrollBar().setValue(pos)
         self.actionPrevDiff.setEnabled(False)
-        connect(self.textView, SIGNAL('filled'),
+        self.textView.filled.connect(
                 lambda self=self: self.actionNextDiff.setEnabled(self.textView.fileMode() and self.textView.nDiffs()))
 
     def goto(self, rev):
@@ -231,6 +222,8 @@ class FileDiffViewer(AbstractFileDialog, _FileDiffViewer):
     """
     _uifile = 'filediffviewer.ui'
 
+    diff_filled = pyqtSignal()
+
     def setupViews(self):
         self.tableView_revisions = self.tableView_revisions_left
         self.tableViews = {'left': self.tableView_revisions_left,
@@ -246,6 +239,7 @@ class FileDiffViewer(AbstractFileDialog, _FileDiffViewer):
         for side, idx  in (('left', 0), ('right', 3)):
             sci = Qsci.QsciScintilla(self.frame)
             sci.setFont(self._font)
+            sci.setUtf8(True)
             sci.verticalScrollBar().setFocusPolicy(Qt.StrongFocus)
             sci.setFocusProxy(sci.verticalScrollBar())
             sci.verticalScrollBar().installEventFilter(self)
@@ -281,43 +275,43 @@ class FileDiffViewer(AbstractFileDialog, _FileDiffViewer):
             self.diffblock.linkScrollBar(sci.verticalScrollBar(), side)
             lay.insertWidget(idx, blk)
             self.block[side] = blk
+
         lay.insertWidget(2, self.diffblock)
 
         for side in sides:
             table = getattr(self, 'tableView_revisions_%s' % side)
             table.setTabKeyNavigation(False)
             #table.installEventFilter(self)
-            connect(table, SIGNAL('revisionSelected'), self.revisionSelected)
-            connect(table, SIGNAL('revisionActivated'), self.revisionActivated)
+            table.revision_selected.connect(self.revisionSelected)
+            table.revision_selected[int].connect(self.revisionSelected)
+            table.revision_activated.connect(self.revisionActivated)
+            table.revision_activated[int].connect(self.revisionActivated)
 
-            connect(self.viewers[side].verticalScrollBar(),
-                    QtCore.SIGNAL('valueChanged(int)'),
+            self.viewers[side].verticalScrollBar().valueChanged[int].connect(
                     lambda value, side=side: self.vbar_changed(value, side))
             self.attachQuickBar(table.goto_toolbar)
 
         self.setTabOrder(table, self.viewers['left'])
         self.setTabOrder(self.viewers['left'], self.viewers['right'])
+        self.setWindowTitle('hgview diff: %s' % os.path.abspath(self.filename))
 
         # timer used to fill viewers with diff block markers during GUI idle time
         self.timer = QtCore.QTimer()
         self.timer.setSingleShot(False)
-        connect(self.timer, SIGNAL("timeout()"),
-                self.idle_fill_files)
+        self.timer.timeout.connect(self.idle_fill_files)
 
     def setupModels(self):
         self.filedata = {'left': None, 'right': None}
         self._invbarchanged = False
         self.filerevmodel = FileRevModel(self.repo, self.filename)
-        connect(self.filerevmodel, QtCore.SIGNAL('filled'),
-                self.modelFilled)
+        self.filerevmodel.filled.connect(self.modelFilled)
         self.tableView_revisions_left.setModel(self.filerevmodel)
         self.tableView_revisions_right.setModel(self.filerevmodel)
 
     def createActions(self):
-        connect(self.actionClose, SIGNAL('triggered()'),
-                self.close)
-        connect(self.actionReload, SIGNAL('triggered()'),
-                self.reload)
+        self.actionClose.triggered.connect(self.close)
+        self.addAction(self.actionClose)
+        self.actionReload.triggered.connect(self.reload)
         self.actionClose.setIcon(geticon('quit'))
         self.actionReload.setIcon(geticon('reload'))
 
@@ -325,10 +319,8 @@ class FileDiffViewer(AbstractFileDialog, _FileDiffViewer):
         self.actionNextDiff.setShortcut('Alt+Down')
         self.actionPrevDiff = QtGui.QAction(geticon('up'), 'Previous diff', self)
         self.actionPrevDiff.setShortcut('Alt+Up')
-        connect(self.actionNextDiff, SIGNAL('triggered()'),
-                self.nextDiff)
-        connect(self.actionPrevDiff, SIGNAL('triggered()'),
-                self.prevDiff)
+        self.actionNextDiff.triggered.connect(self.nextDiff)
+        self.actionPrevDiff.triggered.connect(self.prevDiff)
         self.actionNextDiff.setEnabled(False)
         self.actionPrevDiff.setEnabled(False)
 
@@ -338,8 +330,7 @@ class FileDiffViewer(AbstractFileDialog, _FileDiffViewer):
         self.toolBar_edit.addAction(self.actionPrevDiff)
 
     def modelFilled(self):
-        disconnect(self.filerevmodel, SIGNAL('filled'),
-                   self.modelFilled)
+        self.filerevmodel.filled.disconnect(self.modelFilled)
         if self._show_rev is not None:
             rev = self._show_rev
             self._show_rev = None
@@ -354,7 +345,7 @@ class FileDiffViewer(AbstractFileDialog, _FileDiffViewer):
             side = 'left'
         path = self.filerevmodel.graph.nodesdict[rev].extra[0]
         fc = self.repo.changectx(rev).filectx(path)
-        self.filedata[side] = fc.data().splitlines()
+        self.filedata[side] = tounicode(fc.data()).splitlines()
         self.update_diff(keeppos=otherside[side])
 
     def goto(self, rev):
@@ -401,7 +392,7 @@ class FileDiffViewer(AbstractFileDialog, _FileDiffViewer):
                 self._diff = None
                 self.timer.stop()
                 self.setDiffNavActions(-1)
-                self.emit(SIGNAL('diffFilled'))
+                self.diff_filled.emit()
                 break
 
             tag, alo, ahi, blo, bhi = self._diff.get_opcodes().pop(0)
