@@ -15,30 +15,22 @@
 """
 Qt4 high level widgets for hg repo changelogs and filelogs
 """
-import sys
 import difflib
+from itertools import imap
 
-from mercurial.node import hex, short as short_hex, bin as short_bin
-from mercurial import util
 try:
-    from mercurial.error import LookupError
+    from mercurial.error import LookupError as HgLookupError
 except ImportError:
-    from mercurial.revlog import LookupError
+    from mercurial.revlog import LookupError as HgLookupError
 
 from PyQt4 import QtCore, QtGui, Qsci
-Qt = QtCore.Qt
-connect = QtCore.QObject.connect
-SIGNAL = QtCore.SIGNAL
-nullvariant = QtCore.QVariant()
+from PyQt4.QtCore import Qt, pyqtSignal
 
-from hgviewlib.decorators import timeit
-from hgviewlib.util import exec_flag_changed, isbfile, bfilepath
+from hgviewlib.util import exec_flag_changed, isbfile, bfilepath, tounicode
 from hgviewlib.config import HgConfig
 
 from hgviewlib.qt4 import icon as geticon
 from hgviewlib.qt4.hgfiledialog import FileViewer, FileDiffViewer
-from hgviewlib.qt4.hgmanifestdialog import ManifestViewer
-from hgviewlib.qt4.quickbar import QuickBar
 from hgviewlib.qt4.lexers import get_lexer
 from hgviewlib.qt4.blockmatcher import BlockList
 
@@ -76,31 +68,61 @@ class Annotator(qsci):
             self.SendScintilla(qsci.SCI_MARKERSETBACK, marker, color)
             self.markers.append(marker)
 
-        connect(textarea.verticalScrollBar(),
-                SIGNAL('valueChanged(int)'),
+        textarea.verticalScrollBar().valueChanged[int].connect(
                 self.verticalScrollBar().setValue)
 
     def setFilectx(self, fctx):
-        self.fctx = fctx
-        ann = [f.rev() for f, line in fctx.annotate(follow=True)]
-        self.setText('\n'.join(map(str, ann)))
+        ann = [f.rev() for f, __ in fctx.annotate(follow=True)]
+        self.setText('\n'.join(imap(str, ann)))
         allrevs = list(sorted(set(ann)))
         for i, rev in enumerate(ann):
             idx = allrevs.index(rev)
             self.markerAdd(i, self.markers[idx % len(self.markers)])
 
 
-
 class HgQsci(qsci):
 
     def __init__(self, *args, **kwargs):
         super(HgQsci, self).__init__(*args, **kwargs)
+        super(HgQsci, self).setUtf8(True)
+        self.actions = {}
         self.createActions()
+        self.setFrameStyle(0)
+
+        self.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOn)
+        self.setReadOnly(True)
+
+        self.SendScintilla(qsci.SCI_SETCARETSTYLE, 0)
+
+        # margin 1 is used for line numbers
+        self.setMarginLineNumbers(1, True)
+        self.setMarginWidth(1, '000')
+
+        self.SendScintilla(qsci.SCI_INDICSETSTYLE, 8, qsci.INDIC_ROUNDBOX)
+        self.SendScintilla(qsci.SCI_INDICSETUNDER, 8, True)
+        self.SendScintilla(qsci.SCI_INDICSETFORE, 8, 0xBBFFFF)
+        self.SendScintilla(qsci.SCI_INDICSETSTYLE, 9, qsci.INDIC_ROUNDBOX)
+        self.SendScintilla(qsci.SCI_INDICSETUNDER, 9, True)
+        self.SendScintilla(qsci.SCI_INDICSETFORE, 9, 0x58A8FF)
+
+        # hide margin 0 (markers)
+        self.SendScintilla(qsci.SCI_SETMARGINTYPEN, 0, 0)
+        self.SendScintilla(qsci.SCI_SETMARGINWIDTHN, 0, 0)
+
+        # define markers for colorize zones of diff
+        self.markerplus = self.markerDefine(qsci.Background)
+        self.SendScintilla(qsci.SCI_MARKERSETBACK, self.markerplus, 0xB0FFA0)
+        self.markerminus = self.markerDefine(qsci.Background)
+        self.SendScintilla(qsci.SCI_MARKERSETBACK, self.markerminus, 0xA0A0FF)
+        self.markertriangle = self.markerDefine(qsci.Background)
+        self.SendScintilla(qsci.SCI_MARKERSETBACK, self.markertriangle, 0xFFA0A0)
 
     def _action_defs(self):
         return [
             ("diffmode", self.tr("Diff mode"), 'diffmode' ,
              self.tr('Enable/Disable Diff mode'), None, None),
+            ("ignorews", self.tr("Ignore all space"), None ,
+             self.tr('Enable/Disable white space checking when comparing lines'), None, None),
             ("annmode", self.tr("Annotate mode"), None,
              self.tr('Enable/Disable Annotate mode'), None, None),
             ("next", self.tr('Next hunk'), 'down', self.tr('Jump to the next hunk'),
@@ -115,7 +137,7 @@ class HgQsci(qsci):
         ]
 
     def createActions(self):
-        self._actions = {}
+        self.actions = {}
         for name, desc, icon, tip, key, cb in self._action_defs():
             act = QtGui.QAction(desc, self)
             if icon:
@@ -125,79 +147,91 @@ class HgQsci(qsci):
             if key:
                 act.setShortcut(key)
             if cb:
-                connect(act, SIGNAL('triggered()'), cb)
-            self._actions[name] = act
+                act.triggered.connect(cb)
+            self.actions[name] = act
             self.addAction(act)
-        self._actions['diffmode'].setCheckable(True)
-        self._actions['annmode'].setCheckable(True)
-        self._actions['show-big-file'].setCheckable(True)
+        self.actions['diffmode'].setCheckable(True)
+        self.actions['ignorews'].setCheckable(True)
+        self.actions['annmode'].setCheckable(True)
+        self.actions['show-big-file'].setCheckable(True)
 
     def contextMenuEvent(self, event):
         menu = QtGui.QMenu(self)
         for act in [None, 'diffmode', 'prev', 'next',
-                    None, 'show-big-file']:
+                    None, 'show-big-file', None, 'ignorews']:
             if act:
-                menu.addAction(self._actions[act])
+                menu.addAction(self.actions[act])
             else:
                 menu.addSeparator()
         menu.exec_(event.globalPos())
 
+    def clear_highlights(self):
+        n = self.length()
+        self.SendScintilla(qsci.SCI_SETINDICATORCURRENT, 8) # highlight
+        self.SendScintilla(qsci.SCI_INDICATORCLEARRANGE, 0, n)
+        self.SendScintilla(qsci.SCI_SETINDICATORCURRENT, 9) # current found
+                                                            # occurrence
+        self.SendScintilla(qsci.SCI_INDICATORCLEARRANGE, 0, n)
+
+    def highlight_current_search_string(self, pos, text):
+        line = self.SendScintilla(qsci.SCI_LINEFROMPOSITION, pos)
+        self.ensureLineVisible(line)
+        self.SendScintilla(qsci.SCI_SETINDICATORCURRENT, 9)
+        self.SendScintilla(qsci.SCI_INDICATORCLEARRANGE, 0, pos)
+        self.SendScintilla(qsci.SCI_INDICATORFILLRANGE, pos, len(text))
+
+    def search_and_highlight_string(self, text):
+        data = unicode(self.text())
+        self.SendScintilla(qsci.SCI_SETINDICATORCURRENT, 8)
+        pos = [data.find(text)]
+        n = len(text)
+        while pos[-1] > -1:
+            self.SendScintilla(qsci.SCI_INDICATORFILLRANGE, pos[-1], n)
+            pos.append(data.find(text, pos[-1]+1))
+        return [x for x in pos if x > -1]
+
 
 class HgFileView(QtGui.QFrame):
+
+    message_logged = pyqtSignal(str, int)
+    rev_for_diff_changed = pyqtSignal(int)
+
     def __init__(self, parent=None):
+        self._diff = None
+        self._diffs = None
+        self.cfg = None
         QtGui.QFrame.__init__(self, parent)
         framelayout = QtGui.QVBoxLayout(self)
-        framelayout.setContentsMargins(0,0,0,0)
+        framelayout.setContentsMargins(0, 0, 0, 0)
         framelayout.setSpacing(0)
 
-        l = QtGui.QHBoxLayout()
-        l.setContentsMargins(0,0,0,0)
-        l.setSpacing(0)
-
-        self.topLayout = QtGui.QVBoxLayout()
+        self.info_frame = QtGui.QFrame()
+        framelayout.addWidget(self.info_frame)
+        l = QtGui.QVBoxLayout()
+        self.info_frame.setLayout(l)
         self.filenamelabel = QtGui.QLabel()
         self.filenamelabel.setWordWrap(True)
         self.filenamelabel.setTextInteractionFlags(
             QtCore.Qt.TextSelectableByKeyboard|
-            QtCore.Qt.TextSelectableByMouse)
+            QtCore.Qt.TextSelectableByMouse|
+            QtCore.Qt.LinksAccessibleByMouse)
+        self.filenamelabel.linkActivated.connect(
+            lambda link: self.displayFile(show_big_file=True))
         self.execflaglabel = QtGui.QLabel()
         self.execflaglabel.setWordWrap(True)
-        self.topLayout.addWidget(self.filenamelabel)
-        self.topLayout.addWidget(self.execflaglabel)
+        l.addWidget(self.filenamelabel)
+        l.addWidget(self.execflaglabel)
         self.execflaglabel.hide()
-        framelayout.addLayout(self.topLayout)
-        framelayout.addLayout(l, 1)
+
+        self.filedata_frame = QtGui.QFrame()
+        framelayout.addWidget(self.filedata_frame)
+        l = QtGui.QHBoxLayout()
+        l.setContentsMargins(0,0,0,0)
+        l.setSpacing(0)
+        self.filedata_frame.setLayout(l)
 
         self.sci = HgQsci(self)
-        self.sci.setFrameStyle(0)
         l.addWidget(self.sci, 1)
-        self.sci.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOn)
-        self.sci.setReadOnly(True)
-
-        self.sci.SendScintilla(qsci.SCI_SETCARETSTYLE, 0)
-
-        # margin 1 is used for line numbers
-        self.sci.setMarginLineNumbers(1, True)
-        self.sci.setMarginWidth(1, '000')
-
-        self.sci.SendScintilla(qsci.SCI_INDICSETSTYLE, 8, qsci.INDIC_ROUNDBOX)
-        self.sci.SendScintilla(qsci.SCI_INDICSETUNDER, 8, True)
-        self.sci.SendScintilla(qsci.SCI_INDICSETFORE, 8, 0xBBFFFF)
-        self.sci.SendScintilla(qsci.SCI_INDICSETSTYLE, 9, qsci.INDIC_ROUNDBOX)
-        self.sci.SendScintilla(qsci.SCI_INDICSETUNDER, 9, True)
-        self.sci.SendScintilla(qsci.SCI_INDICSETFORE, 9, 0x58A8FF)
-
-        # hide margin 0 (markers)
-        self.sci.SendScintilla(qsci.SCI_SETMARGINTYPEN, 0, 0)
-        self.sci.SendScintilla(qsci.SCI_SETMARGINWIDTHN, 0, 0)
-
-        # define markers for colorize zones of diff
-        self.markerplus = self.sci.markerDefine(qsci.Background)
-        self.sci.SendScintilla(qsci.SCI_MARKERSETBACK, self.markerplus, 0xB0FFA0)
-        self.markerminus = self.sci.markerDefine(qsci.Background)
-        self.sci.SendScintilla(qsci.SCI_MARKERSETBACK, self.markerminus, 0xA0A0FF)
-        self.markertriangle = self.sci.markerDefine(qsci.Background)
-        self.sci.SendScintilla(qsci.SCI_MARKERSETBACK, self.markertriangle, 0xFFA0A0)
 
         ll = QtGui.QVBoxLayout()
         ll.setContentsMargins(0, 0, 0, 0)
@@ -233,19 +267,15 @@ class HgFileView(QtGui.QFrame):
 
         self.timer = QtCore.QTimer()
         self.timer.setSingleShot(False)
-        self.connect(self.timer, QtCore.SIGNAL("timeout()"),
-                     self.idle_fill_files)
-        self.connect(self.sci._actions['diffmode'], SIGNAL('toggled(bool)'),
-                     self.setMode)
-        self.connect(self.sci._actions['annmode'], SIGNAL('toggled(bool)'),
-                     self.setAnnotate)
-        self.connect(self.sci._actions['prev'], SIGNAL('triggered()'),
-                     self.prevDiff)
-        self.connect(self.sci._actions['next'], SIGNAL('triggered()'),
-                     self.nextDiff)
-        self.connect(self.sci._actions['show-big-file'], SIGNAL('toggled(bool)'),
-                     self.showBigFile)
-        self.sci._actions['diffmode'].setChecked(True)
+        self.timer.timeout.connect(self.idle_fill_files)
+        self.sci.actions['diffmode'].toggled[bool].connect(self.setMode)
+        self.sci.actions['ignorews'].toggled[bool].connect(
+            lambda value: self.setUiConfig('diff', 'ignorews', value))
+        self.sci.actions['annmode'].toggled[bool].connect(self.setAnnotate)
+        self.sci.actions['prev'].triggered.connect(self.prevDiff)
+        self.sci.actions['next'].triggered.connect(self.nextDiff)
+        self.sci.actions['show-big-file'].toggled[bool].connect(self.showBigFile)
+        self.sci.actions['diffmode'].setChecked(True)
 
     def resizeEvent(self, event):
         QtGui.QFrame.resizeEvent(self, event)
@@ -270,9 +300,9 @@ class HgFileView(QtGui.QFrame):
             mode = ['file', 'diff'][mode]
         assert mode in ('diff', 'file')
 
-        self.sci._actions['annmode'].setEnabled(not mode)
-        self.sci._actions['next'].setEnabled(not mode)
-        self.sci._actions['prev'].setEnabled(not mode)
+        self.sci.actions['annmode'].setEnabled(not mode)
+        self.sci.actions['next'].setEnabled(not mode)
+        self.sci.actions['prev'].setEnabled(not mode)
         if mode != self._mode:
             self._mode = mode
             self.blk.setVisible(self._mode == 'file')
@@ -280,6 +310,12 @@ class HgFileView(QtGui.QFrame):
 
             self.displayFile()
 
+    def setUiConfig(self, section, name, value):
+        if self._model.repo.ui._tcfg.get(section, name) == value:
+            return
+        self._model.repo.ui._tcfg.set(section, name, value, source='hgview')
+        self.displayFile()
+
     def setAnnotate(self, ann):
         self._annotate = ann
         if ann:
@@ -293,7 +329,8 @@ class HgFileView(QtGui.QFrame):
             is_show_big_file = self._model.graph.maxfilesize < 0
         else:
             is_show_big_file = self.cfg.getMaxFileSize()
-        self.sci._actions['show-big-file'].setChecked(is_show_big_file)
+        self.sci.actions['show-big-file'].setChecked(is_show_big_file)
+        self.sci.actions['ignorews'].setChecked(model.repo.ui.configbool('diff', 'ignorews'))
         self.sci.clear()
 
     def setContext(self, ctx):
@@ -311,7 +348,7 @@ class HgFileView(QtGui.QFrame):
         if rev != self._p_rev:
             self.displayFile(rev=rev)
 
-    def displayFile(self, filename=None, rev=None):
+    def displayFile(self, filename=None, rev=None, show_big_file=None):
         if filename is None:
             filename = self._filename
 
@@ -323,7 +360,7 @@ class HgFileView(QtGui.QFrame):
 
         if rev is not None:
             self._p_rev = rev
-            self.emit(SIGNAL('revForDiffChanged'), rev)
+            self.rev_for_diff_changed.emit(rev)
         self.sci.clear()
         self.ann.clear()
         self.filenamelabel.setText(" ")
@@ -333,13 +370,29 @@ class HgFileView(QtGui.QFrame):
         try:
             filectx = self._ctx.filectx(self._realfilename)
 
-        except LookupError: # occur on deleted files
+        except HgLookupError: # occur on deleted files
             return
         if self._mode == 'diff' and self._p_rev is not None:
             mode = self._p_rev
         else:
             mode = self._mode
-        flag, data = self._model.graph.filedata(filename, self._ctx.rev(), mode)
+        if show_big_file:
+            flag, data = self._model.graph.filedata(filename, self._ctx.rev(), mode, maxfilesize=-1)
+        else:
+            flag, data = self._model.graph.filedata(filename, self._ctx.rev(), mode)
+        if flag == 'file too big':
+            self.filedata_frame.hide()
+            message = (('<center>'
+                        'File size (%s) greater than configured maximum value: '
+                        '<font color="red"> maxfilesize=%i</font><br>'
+                        '<br>'
+                        '<a href="show-big-file">Click to display anyway '
+                        '<img src=":/icons/heavy_small.png" width="16" height="16"></a>.'
+                        '</center>') % (data, self.cfg.getMaxFileSize()))
+            self.filenamelabel.setText(message)
+            return
+        else:
+            self.filedata_frame.show()
         if flag == '-':
             return
         if flag == '':
@@ -351,34 +404,34 @@ class HgFileView(QtGui.QFrame):
             self.sci.setMarginWidth(1, str(nlines)+'0')
         self.sci.setLexer(lexer)
         self._cur_lexer = lexer
-        if data not in ('file too big', 'binary file'):
+        if data not in (u'file too big', u'binary file'):
             self.filedata = data
         else:
             self.filedata = None
 
         flag = exec_flag_changed(filectx)
         if flag:
-            self.execflaglabel.setText("<b>exec mode has been <font color='red'>%s</font></b>" % flag)
+            self.execflaglabel.setText(u"<b>exec mode has been <font color='red'>%s</font></b>" % flag)
             self.execflaglabel.show()
         else:
             self.execflaglabel.hide()
 
-        labeltxt = ''
+        labeltxt = u''
         if isbfile(self._realfilename):
-            labeltxt += '[bfile tracked] '
-        labeltxt += "<b>%s</b>" % self._filename
+            labeltxt += u'[bfile tracked] '
+        labeltxt += u"<b>%s</b>" % tounicode(self._filename)
 
         if self._p_rev is not None:
-            labeltxt += ' (diff from rev %s)' % self._p_rev
+            labeltxt += u' (diff from rev %s)' % self._p_rev
         renamed = filectx.renamed()
         if renamed:
-            labeltxt += ' <i>(renamed from %s)</i>' % bfilepath(renamed[0])
+            labeltxt += u' <i>(renamed from %s)</i>' % tounicode(bfilepath(renamed[0]))
         self.filenamelabel.setText(labeltxt)
 
         self.sci.setText(data)
         if self._find_text:
             self.highlightSearchString(self._find_text)
-        self.sci._actions['prev'].setEnabled(False)
+        self.sci.actions['prev'].setEnabled(False)
         self.updateDiffDecorations()
         if self._mode == 'file' and self._annotate:
             if filectx.rev() is None: # XXX hide also for binary files
@@ -407,7 +460,7 @@ class HgFileView(QtGui.QFrame):
                 return
             m = self._ctx.filectx(self._filename).renamed()
             if m:
-                pfilename, pnode = m
+                pfilename, __ = m
             else:
                 pfilename = self._filename
             _, parentdata = self._model.graph.filedata(pfilename,
@@ -424,8 +477,9 @@ class HgFileView(QtGui.QFrame):
 
     def _nextDiff(self):
         if self._mode == 'file':
-            row, column = self.sci.getCursorPosition()
-            for i, (lo, hi) in enumerate(self._diffs):
+            row, __ = self.sci.getCursorPosition()
+            lo = 0
+            for i, (lo, __) in enumerate(self._diffs):
                 if lo > row:
                     last = (i == (len(self._diffs)-1))
                     break
@@ -437,12 +491,13 @@ class HgFileView(QtGui.QFrame):
 
     def nextDiff(self):
         notlast = self._nextDiff()
-        self.sci._actions['next'].setEnabled(self.fileMode() and notlast and self.nDiffs())
-        self.sci._actions['prev'].setEnabled(self.fileMode() and self.nDiffs())
+        self.sci.actions['next'].setEnabled(self.fileMode() and notlast and self.nDiffs())
+        self.sci.actions['prev'].setEnabled(self.fileMode() and self.nDiffs())
 
     def _prevDiff(self):
         if self._mode == 'file':
-            row, column = self.sci.getCursorPosition()
+            row, __ = self.sci.getCursorPosition()
+            lo = 0
             for i, (lo, hi) in enumerate(reversed(self._diffs)):
                 if hi < row:
                     first = (i == (len(self._diffs)-1))
@@ -455,10 +510,8 @@ class HgFileView(QtGui.QFrame):
 
     def prevDiff(self):
         notfirst = self._prevDiff()
-        self.sci._actions['prev'].setEnabled(self.fileMode() and notfirst and self.nDiffs())
-        self.sci._actions['next'].setEnabled(self.fileMode() and self.nDiffs())
-
-
+        self.sci.actions['prev'].setEnabled(self.fileMode() and notfirst and self.nDiffs())
+        self.sci.actions['next'].setEnabled(self.fileMode() and self.nDiffs())
 
     def nextLine(self):
         x, y = self.sci.getCursorPosition()
@@ -486,82 +539,54 @@ class HgFileView(QtGui.QFrame):
 
     def searchString(self, text):
         self._find_text = text
-        self.clearHighlights()
+        self.sci.clear_highlights()
         findpos = self.highlightSearchString(self._find_text)
         if findpos:
             def finditer(self, findpos):
                 if self._find_text:
                     for pos in findpos:
-                        self.highlightCurrentSearchString(pos, self._find_text)
+                        self.sci.highlight_current_search_string(pos, self._find_text)
                         yield self._ctx.rev(), self._filename, pos
             return finditer(self, findpos)
 
-    def clearHighlights(self):
-        n = self.sci.length()
-        self.sci.SendScintilla(qsci.SCI_SETINDICATORCURRENT, 8) # highlight
-        self.sci.SendScintilla(qsci.SCI_INDICATORCLEARRANGE, 0, n)
-        self.sci.SendScintilla(qsci.SCI_SETINDICATORCURRENT, 9) # current found occurrence
-        self.sci.SendScintilla(qsci.SCI_INDICATORCLEARRANGE, 0, n)
-
     def highlightSearchString(self, text):
-        data = unicode(self.sci.text())
-        self.sci.SendScintilla(qsci.SCI_SETINDICATORCURRENT, 8)
-        pos = [data.find(text)]
-        n = len(text)
-        while pos[-1] > -1:
-            self.sci.SendScintilla(qsci.SCI_INDICATORFILLRANGE, pos[-1], n)
-            pos.append(data.find(text, pos[-1]+1))
-        pos = [x for x in pos if x > -1]
-        self.emit(SIGNAL('showMessage'),
-                  "Found %d occurrences of '%s' in current file or diff" % (len(pos), text),
-                  2000)
+        pos = self.sci.search_and_highlight_string(text)
+        msg = u"Found %d occurrences of '%s' in current file or diff" % \
+              (len(pos), tounicode(text))
+        self.message_logged.emit(msg, 2000)
         return pos
 
-    def highlightCurrentSearchString(self, pos, text):
-        line = self.sci.SendScintilla(qsci.SCI_LINEFROMPOSITION, pos)
-        #line, idx = w.lineIndexFromPosition(nextpos)
-        self.sci.ensureLineVisible(line)
-        self.sci.SendScintilla(qsci.SCI_SETINDICATORCURRENT, 9)
-        self.sci.SendScintilla(qsci.SCI_INDICATORCLEARRANGE, 0, pos)
-        self.sci.SendScintilla(qsci.SCI_INDICATORFILLRANGE, pos, len(text))
-
     def verticalScrollBar(self):
         return self.sci.verticalScrollBar()
 
-
     def idle_fill_files(self):
         # we make a burst of diff-lines computed at once, but we
         # disable GUI updates for efficiency reasons, then only
         # refresh GUI at the end of the burst
         self.sci.setUpdatesEnabled(False)
         self.blk.setUpdatesEnabled(False)
-        for n in range(30): # burst pool
+        for __ in range(30): # burst pool
             if self._diff is None or not self._diff.get_opcodes():
                 self._diff = None
                 self.timer.stop()
-                self.sci._actions['next'].setEnabled(self.fileMode() and self.nDiffs())
+                self.sci.actions['next'].setEnabled(self.fileMode() and self.nDiffs())
                 break
 
-            tag, alo, ahi, blo, bhi = self._diff.get_opcodes().pop(0)
+            tag, __, __, blo, bhi = self._diff.get_opcodes().pop(0)
             if tag == 'replace':
                 self._diffs.append([blo, bhi])
                 self.blk.addBlock('x', blo, bhi)
                 for i in range(blo, bhi):
-                    self.sci.markerAdd(i, self.markertriangle)
+                    self.sci.markerAdd(i, self.sci.markertriangle)
 
             elif tag == 'delete':
                 pass
-                # self.block['left'].addBlock('-', alo, ahi)
-                # self.diffblock.addBlock('-', alo, ahi, blo, bhi)
-                # w = self.viewers['left']
-                # for i in range(alo, ahi):
-                #     w.markerAdd(i, self.markerminus)
 
             elif tag == 'insert':
                 self._diffs.append([blo, bhi])
                 self.blk.addBlock('+', blo, bhi)
                 for i in range(blo, bhi):
-                    self.sci.markerAdd(i, self.markerplus)
+                    self.sci.markerAdd(i, self.sci.markerplus)
 
             elif tag == 'equal':
                 pass
@@ -578,6 +603,9 @@ class HgFileListView(QtGui.QTableView):
     """
     A QTableView for displaying a HgFileListModel
     """
+
+    file_selected = pyqtSignal([str, int], [str])
+
     def __init__(self, parent=None):
         QtGui.QTableView.__init__(self, parent)
         self.setShowGrid(False)
@@ -592,24 +620,19 @@ class HgFileListView(QtGui.QTableView):
 
         self.createActions()
 
-        connect(self.horizontalHeader(), SIGNAL('sectionDoubleClicked(int)'),
+        self.horizontalHeader().sectionDoubleClicked[int].connect(
                 self.toggleFullFileList)
-        connect(self,
-                SIGNAL('doubleClicked (const QModelIndex &)'),
-                self.fileActivated)
+        self.doubleClicked.connect(self.fileActivated)
 
-        connect(self.horizontalHeader(),
-                SIGNAL('sectionResized(int, int, int)'),
+        self.horizontalHeader().sectionResized[int, int, int].connect(
                 self.sectionResized)
         self._diff_dialogs = {}
         self._nav_dialogs = {}
 
     def setModel(self, model):
         QtGui.QTableView.setModel(self, model)
-        connect(model, SIGNAL('layoutChanged()'),
-                self.fileSelected)
-        connect(self.selectionModel(),
-                SIGNAL('currentRowChanged (const QModelIndex & , const QModelIndex & )'),
+        model.layoutChanged.connect(self.fileSelected)
+        self.selectionModel().currentRowChanged.connect(
                 self.fileSelected)
         self.horizontalHeader().setResizeMode(1, QtGui.QHeaderView.Stretch)
 
@@ -622,7 +645,11 @@ class HgFileListView(QtGui.QTableView):
             index = self.currentIndex()
         sel_file = self.model().fileFromIndex(index)
         from_rev = self.model().revFromIndex(index)
-        self.emit(SIGNAL('fileSelected'), sel_file, from_rev)
+        # signal get unicode as input
+        if from_rev is None:
+            self.file_selected[str].emit(tounicode(sel_file))
+        else:
+            self.file_selected[str, int].emit(tounicode(sel_file), from_rev)
 
     def selectFile(self, filename):
         self.setCurrentIndex(self.model().indexFromFile(filename))
@@ -681,7 +708,7 @@ class HgFileListView(QtGui.QTableView):
             if key:
                 act.setShortcut(key)
             if cb:
-                connect(act, SIGNAL('triggered()'), cb)
+                act.triggered.connect(cb)
             self._actions[name] = act
             self.addAction(act)
 
