@@ -34,11 +34,6 @@ from hgviewlib.qt4.hgdialogmixin import HgDialogMixin
 from hgviewlib.qt4.hgrepomodel import ManifestModel
 from hgviewlib.qt4.lexers import get_lexer
 
-connect = QtCore.QObject.connect
-disconnect = QtCore.QObject.disconnect
-SIGNAL = QtCore.SIGNAL
-nullvariant = QtCore.QVariant()
-
 
 class ManifestViewer(QtGui.QMainWindow, HgDialogMixin, _ManifestViewer):
     """
@@ -49,7 +44,7 @@ class ManifestViewer(QtGui.QMainWindow, HgDialogMixin, _ManifestViewer):
         self.repo = repo
         QtGui.QMainWindow.__init__(self)
         HgDialogMixin.__init__(self)
-        self.setWindowTitle('Hg manifest viewer - %s:%s' % (repo.root, noderev))
+        self.setWindowTitle('hgview manifest: %s revision %s' % (repo.root, noderev))
 
         # hg repo
         self.repo = repo
@@ -66,14 +61,13 @@ class ManifestViewer(QtGui.QMainWindow, HgDialogMixin, _ManifestViewer):
     def setupModels(self):
         self.treemodel = ManifestModel(self.repo, self.rev)
         self.treeView.setModel(self.treemodel)
-        connect(self.treeView.selectionModel(),
-                SIGNAL('currentChanged(const QModelIndex &, const QModelIndex &)'),
-                self.fileSelected)
+        self.treeView.selectionModel().currentChanged.connect(
+            self.fileSelected)
 
     def createActions(self):
-        connect(self.actionClose, SIGNAL('triggered()'),
-                self.close)
+        self.actionClose.triggered.connect(self.close)
         self.actionClose.setIcon(geticon('quit'))
+        self.addAction(self.actionClose)
 
     def setupTextview(self):
         lay = QtGui.QHBoxLayout(self.mainFrame)
@@ -85,6 +79,7 @@ class ManifestViewer(QtGui.QMainWindow, HgDialogMixin, _ManifestViewer):
         sci.setMarginWidth(1, '000')
         sci.setReadOnly(True)
         sci.setFont(self._font)
+        sci.setUtf8(True)
 
         sci.SendScintilla(sci.SCI_SETSELEOLFILLED, True)
         self.textView = sci
@@ -102,12 +97,12 @@ class ManifestViewer(QtGui.QMainWindow, HgDialogMixin, _ManifestViewer):
             return
 
         if fc.size() > self.max_file_size:
-            data = "file too big"
+            data = u"file too big"
         else:
             # return the whole file
             data = fc.data()
             if util.binary(data):
-                data = "binary file"
+                data = u"binary file"
             else:
                 data = tounicode(data)
                 lexer = get_lexer(path, data)
