@@ -18,6 +18,7 @@ Qt4 model for hg repo changelogs and filelogs
 import sys
 import re
 import os, os.path as osp
+from functools import partial
 
 from mercurial.node import nullrev
 from mercurial.node import hex, short as short_hex
@@ -27,55 +28,54 @@ from mercurial import util, error
 from hgviewlib.hggraph import Graph, ismerge, diff as revdiff, HgRepoListWalker
 from hgviewlib.hggraph import revision_grapher, filelog_grapher, getlog, gettags
 from hgviewlib.config import HgConfig
-from hgviewlib.util import tounicode, isbfile, Curry
+from hgviewlib.util import tounicode, isbfile, xml_escape, allbranches
 from hgviewlib.qt4 import icon as geticon
 from hgviewlib.decorators import timeit
 from hgviewlib.hgpatches import phases
 
-from PyQt4 import QtCore, QtGui
-connect = QtCore.QObject.connect
-SIGNAL = QtCore.SIGNAL
-nullvariant = QtCore.QVariant()
+from PyQt4.QtGui import QColor, QPixmap, QPainter, QPen, QFont
+from PyQt4.QtCore import Qt, pyqtSignal, QAbstractItemModel, QAbstractTableModel, \
+     QObject, QDateTime, QTimer, QModelIndex, QPointF
 
 # XXX make this better than a poor hard written list...
 COLORS = [ "blue", "darkgreen", "red", "green", "darkblue", "purple",
-           "cyan", QtCore.Qt.darkYellow, "magenta", "darkred", "darkmagenta",
+           "cyan", Qt.darkYellow, "magenta", "darkred", "darkmagenta",
            "darkcyan", "gray", ]
-COLORS = [str(QtGui.QColor(x).name()) for x in COLORS]
-#COLORS = [str(color) for color in QtGui.QColor.colorNames()]
+COLORS = [str(QColor(x).name()) for x in COLORS]
+#COLORS = [str(color) for color in QColor.colorNames()]
 
 # We use two colors, One for even rows and one for odd rows
-COLOR_BG_OBSOLETE = [QtGui.QColor(255, 250, 250), QtGui.QColor(243, 230, 230)]
-COLOR_BG_TROUBLED = [QtGui.QColor(255, 193,  71), QtGui.QColor(255, 153,  51)]
-COLOR_BG_HIGHLIGHT = [QtGui.QColor(127, 199, 175),
-                      QtGui.QColor(127, 199, 175).lighter()]
-
+COLOR_BG_OBSOLETE = [QColor(255, 250, 250), QColor(243, 230, 230)]
+COLOR_BG_TROUBLED = [QColor(255, 193,  71), QColor(255, 153,  51)]
+COLOR_BG_HIGHLIGHT = [QColor(127, 199, 175),
+                      QColor(127, 199, 175).lighter()]
 
+BOOKMARK_CSS = "color: white; background-color: blue;"
 
 def cvrt_date(date):
     """
     Convert a date given the hg way, ie. couple (date, tz), into a
-    formatted QString
+    formatted string
     """
     if not date:
-        return QtCore.QString(u'')
+        return u''
     date, tzdelay = date
-    return QtCore.QDateTime.fromTime_t(int(date)).toString(QtCore.Qt.LocaleDate)
+    return QDateTime.fromTime_t(int(date)).toString(Qt.LocaleDate)
 
 
 # XXX maybe it's time to make these methods of the model...
 # in following lambdas, ctx is a hg changectx
-_columnmap = {'ID': lambda model, ctx, gnode: ctx.rev() is not None and str(ctx.rev()) or "",
+_columnmap = {'ID': lambda model, ctx, gnode: ctx.rev() is not None and tounicode(ctx.rev()) or u"",
               'Log': getlog,
               'Author': lambda model, ctx, gnode: tounicode(ctx.user()),
               'Date': lambda model, ctx, gnode: cvrt_date(ctx.date()),
               'Tags': gettags,
-              'Branch': lambda model, ctx, gnode: ctx.branch(),
-              'Filename': lambda model, ctx, gnode: gnode.extra[0],
-              'Phase': lambda model, ctx, gnode: ctx.phasestr(),
+              'Branch': lambda model, ctx, gnode: tounicode(ctx.branch()),
+              'Filename': lambda model, ctx, gnode: tounicode(gnode.extra[0]),
+              'Phase': lambda model, ctx, gnode: tounicode(ctx.phasestr()),
               }
 
-_tooltips = {'ID': lambda model, ctx, gnode: ctx.rev() is not None and ctx.hex() or "Working Directory",
+_tooltips = {'ID': lambda model, ctx, gnode: ctx.rev() is not None and tounicode(ctx.hex()) or u"Working Directory",
              }
 
 def auth_width(model, repo):
@@ -85,11 +85,11 @@ def auth_width(model, repo):
     return sorted(auths, cmp=lambda x,y: cmp(len(x), len(y)))[-1]
 
 # in following lambdas, r is a hg repo
+# it return the longuest entry of this column
 _maxwidth = {'ID': lambda self, r: str(len(r.changelog)),
              'Date': lambda self, r: cvrt_date(r.changectx(0).date()),
              'Tags': lambda self, r: sorted(r.tags().keys(), key=len)[-1][:10],
-             'Branch': lambda self, r: sorted(r.branchtags().keys(), key=len)[-1]
-                                              if r.branchtags().keys() else None,
+             'Branch': lambda self, r: ([None] + sorted(allbranches(r), key=len))[-1],
              'Author': lambda self, r: 'author name',
              'Filename': lambda self, r: self.filename,
              'Phase': lambda self, r: sorted(phases.phasenames, key=len)[-1]
@@ -98,26 +98,32 @@ _maxwidth = {'ID': lambda self, r: str(len(r.changelog)),
 def datacached(meth):
     """
     decorator used to cache 'data' method of Qt models. It will *not*
-    cache nullvariant return values (so costly non-null values
+    cache None return values (so costly non-null values
     can be computed and filled as a background process)
     """
     def data(self, index, role):
         if not index.isValid():
-            return nullvariant
+            return None
         row = index.row()
         col = index.column()
         if (row, col, role) in self._datacache:
             return self._datacache[(row, col, role)]
         result = meth(self, index, role)
-        if result is not nullvariant:
+        if result is not None:
             self._datacache[(row, col, role)] = result
         return result
     return data
 
-class HgRepoListModel(QtCore.QAbstractTableModel, HgRepoListWalker):
+
+
+class HgRepoListModel(QAbstractTableModel, HgRepoListWalker):
     """
     Model used for displaying the revisions of a Hg *local* repository
     """
+
+    message_logged = pyqtSignal(str, int)
+    filled = pyqtSignal()
+
     _allcolumns = ('ID', 'Branch', 'Log', 'Author', 'Date', 'Tags',)
     _columns = ('ID', 'Branch', 'Log', 'Author', 'Date', 'Tags',)
     _stretchs = {'Log': 1, }
@@ -128,14 +134,14 @@ class HgRepoListModel(QtCore.QAbstractTableModel, HgRepoListWalker):
         repo is a hg repo instance
         """
         self._fill_timer = None
-        QtCore.QAbstractTableModel.__init__(self, parent)
+        QAbstractTableModel.__init__(self, parent)
         HgRepoListWalker.__init__(self, repo, branch, fromhead, follow, closed=closed)
         self.highlights = []
 
     def setRepo(self, repo, branch='', fromhead=None, follow=False, closed=False):
         HgRepoListWalker.setRepo(self, repo, branch, fromhead, follow, closed=closed)
-        self.emit(SIGNAL('layoutChanged()'))
-        QtCore.QTimer.singleShot(0, Curry(self.emit, SIGNAL('filled')))
+        self.layoutChanged.emit()
+        QTimer.singleShot(0, lambda: self.filled.emit())
         self._fill_timer = self.startTimer(50)
 
     def highlight_rows(self, rows):
@@ -145,13 +151,14 @@ class HgRepoListModel(QtCore.QAbstractTableModel, HgRepoListWalker):
 
     def timerEvent(self, event):
         if event.timerId() == self._fill_timer:
-            self.emit(SIGNAL('showMessage'),
+            self.message_logged.emit(
                       'filling (%s)' % (len(self.graph)),
                       -1)
             if self.graph.isfilled():
                 self.killTimer(self._fill_timer)
                 self._fill_timer = None
-                self.emit(SIGNAL('showMessage'), '', -1)
+                self.updateRowCount()
+                self.message_logged.emit('', -1)
             # we fill the graph data structures without telling
             # views until we are done - this gives
             # maximal GUI responsiveness
@@ -159,13 +166,13 @@ class HgRepoListModel(QtCore.QAbstractTableModel, HgRepoListWalker):
                 self.killTimer(self._fill_timer)
                 self._fill_timer = None
                 self.updateRowCount()
-                self.emit(SIGNAL('showMessage'), '', -1)
+                self.message_logged.emit('', -1)
 
     def updateRowCount(self):
         currentlen = self.rowcount
         newlen = len(self.graph)
         if newlen > self.rowcount:
-            self.beginInsertRows(QtCore.QModelIndex(), currentlen, newlen-1)
+            self.beginInsertRows(QModelIndex(), currentlen, newlen-1)
             self.rowcount = newlen
             self.endInsertRows()
 
@@ -175,7 +182,7 @@ class HgRepoListModel(QtCore.QAbstractTableModel, HgRepoListWalker):
         Return a color at index 'n' rotating in the available
         colors. 'ignore' is a list of colors not to be chosen.
         """
-        ignore = [str(QtGui.QColor(x).name()) for x in ignore]
+        ignore = [str(QColor(x).name()) for x in ignore]
         colors = [x for x in COLORS if x not in ignore]
         if not colors: # ghh, no more available colors...
             colors = COLORS
@@ -187,65 +194,72 @@ class HgRepoListModel(QtCore.QAbstractTableModel, HgRepoListWalker):
         if user in self._users:
             try:
                 color = self._users[user]['color']
-                color = QtGui.QColor(color).name()
+                color = QColor(color).name()
                 self._user_colors[user] = color
             except:
                 pass
         return HgRepoListWalker.user_color(self, user)
 
-    def col2x(self, col, pan):
-        return (self.dot_radius + pan) * col + 2  # max pen width
+    def _display_log(self, ctx, gnode, row):
+        """Display the log column content."""
+        content = []
+        # display bookmarks
+        bookmarks = [tounicode(bookmark) for bookmark in ctx.bookmarks()]
+        if bookmarks:
+            content.extend(u'<span style="white-space: pre; %s"> %s </span>'
+                           % (BOOKMARK_CSS, xml_escape(bkm)) for bkm in bookmarks)
+        # display log
+        style =  "color: grey;" if ctx.obsolete() else u""
+        content.append(u'<span style="white-space: pre; %s"> %s </span>'
+                       % (style, xml_escape(_columnmap['Log'](self, ctx, gnode))))
+        return u' '.join(content)
 
     @datacached
     def data(self, index, role):
         if not index.isValid():
-            return nullvariant
+            return None
         row = index.row()
         self.ensureBuilt(row=row)
         column = self._columns[index.column()]
         gnode = self.graph[row]
         ctx = self.repo.changectx(gnode.rev)
-        if role == QtCore.Qt.DisplayRole:
+        if role == Qt.DisplayRole:
             if column == 'Author': #author
                 user = _columnmap[column](self, ctx, gnode) if ctx.node() else u'' 
-                return QtCore.QVariant(self.user_name(user))
+                return self.user_name(user)
             elif column == 'Log':
-                msg = _columnmap[column](self, ctx, gnode)
-                bookmarks = ctx.bookmarks()
-                if bookmarks:
-                    msg = '<%s> ~ %s' % (','.join(bookmarks), msg)
-                return QtCore.QVariant(msg)
-            return QtCore.QVariant(_columnmap[column](self, ctx, gnode))
-        elif role == QtCore.Qt.ToolTipRole:
-            msg = "<b>Branch:</b> %s<br>\n" % ctx.branch()
-            msg += "<b>Phase:</b> %s<br>\n" % ctx.phasestr()
+                return self._display_log(ctx, gnode, row)
+            return _columnmap[column](self, ctx, gnode)
+        elif role == Qt.ToolTipRole:
+            msg = u"<b>Branch:</b> %s<br>\n" % _columnmap['Branch'](self, ctx, gnode)
+            msg += u"<b>Phase:</b> %s<br>\n" % _columnmap['Phase'](self, ctx, gnode)
             if gnode.rev in self.wd_revs:
-                msg += " <i>Working Directory position"
-                states = 'modified added removed deleted'.split()
+                msg += u" <i>Working Directory position"
+                states = u'modified added removed deleted'.split()
                 status = self.wd_status[self.wd_revs.index(gnode.rev)]
                 status = [state for st, state in zip(status, states) if st]
                 if status:
                     msg += ' (%s)' % (', '.join(status))
-                msg += "</i><br>\n"
+                msg += u"</i><br>\n"
             msg += _tooltips.get(column, _columnmap[column])(self, ctx, gnode)
-            return QtCore.QVariant(msg)
-        elif role == QtCore.Qt.ForegroundRole:
+            return msg
+        elif role == Qt.ForegroundRole:
             color = None
             if column == 'Author': #author
-                user = ctx.user() if ctx.node() else ''
-                color = QtGui.QColor(self.user_color(user))
+                user = tounicode(ctx.user()) if ctx.node() else u''
+                color = QColor(self.user_color(user))
                 if ctx.obsolete():
                     color = color.lighter()
             elif column == 'Branch': #branch
-                color = QtGui.QColor(self.namedbranch_color(ctx.branch()))
+                color = QColor(self.namedbranch_color(ctx.branch()))
                 if ctx.obsolete():
                     color = color.lighter()
             elif ctx.obsolete():
-                color = QtGui.QColor('grey')
+                color = QColor('grey')
             if color is not None:
-                return QtCore.QVariant(color)
+                return color
 
-        elif role == QtCore.Qt.BackgroundRole:
+        elif role == Qt.BackgroundRole:
             row = index.row()
             if row in self.highlights:
                 return COLOR_BG_HIGHLIGHT[row % 2]
@@ -254,117 +268,10 @@ class HgRepoListModel(QtCore.QAbstractTableModel, HgRepoListWalker):
             elif ctx.troubles():
                 return COLOR_BG_TROUBLED[row % 2]
 
-        elif role == QtCore.Qt.DecorationRole:
-            if column == 'Log':
-                if not getattr(ctx, 'applied', True):
-                    return nullvariant
-                radius = self.dot_radius
-                pan = 2
-                w = self.col2x(gnode.cols, pan)
-                h = self.rowheight
-
-                dot_x = self.col2x(gnode.x, pan)
-                dot_y = h / 2
-
-                pix = QtGui.QPixmap(w, h)
-                pix.fill(QtGui.QColor(0,0,0,0))
-                painter = QtGui.QPainter(pix)
-                painter.setRenderHint(QtGui.QPainter.Antialiasing)
-
-                pen = QtGui.QPen(QtCore.Qt.blue)
-                pen.setWidth(2)
-                painter.setPen(pen)
-
-                lpen = QtGui.QPen(pen)
-                lpen.setColor(QtCore.Qt.black)
-                painter.setPen(lpen)
-
-                for y1, y2, lines in ((0, h, gnode.bottomlines),
-                                      (-h, 0, gnode.toplines)):
-                    for start, end, color, fill in lines:
-                        lpen = QtGui.QPen(pen)
-                        color = QtGui.QColor(self.get_color(color))
-                        if not fill:
-                             lpen.setStyle(QtCore.Qt.DotLine)
-                             color.setAlpha(150)
-                        lpen.setColor(color)
-                        lpen.setWidth(2)
-                        painter.setPen(lpen)
-                        x1 = self.col2x(start, pan) + radius / 2
-                        x2 = self.col2x(end, pan) + radius / 2
-                        painter.drawLine(x1, dot_y + y1, x2, dot_y + y2)
-
-                dot_color = QtGui.QColor(self.namedbranch_color(ctx.branch()))
-                dotcolor = QtGui.QColor(dot_color)
-                if ctx.obsolete():
-                    penradius = 1
-                    pencolor = dotcolor.setAlpha(150)
-                elif gnode.rev in self.heads:
-                    penradius = 2
-                    pencolor = dotcolor.darker()
-                else:
-                    penradius = 1
-                    pencolor = QtCore.Qt.black
-
-                dot_y = (h/2) - radius / 2
-
-                painter.setBrush(dotcolor)
-                pen = QtGui.QPen(pencolor)
-                pen.setWidth(penradius)
-                painter.setPen(pen)
-                tags = set(ctx.tags())
-                icn = None
-
-                modified = False
-                atwd = False
-                if gnode.rev in self.wd_revs:
-                    atwd = True
-                    status = self.wd_status[self.wd_revs.index(gnode.rev)]
-                    if [True for st in status if st]:
-                        modified = True
-
-                phase = ctx.phase()
-
-                if gnode.rev is None:
-                    # WD is displayed only if there are local
-                    # modifications, so let's use the modified icon
-                    icn = geticon('modified')
-                elif tags.intersection(self.mqueues):
-                    icn = geticon('mqpatch')
-                #elif modified:
-                #    icn = geticon('modified')
-                elif atwd:
-                    if phase > phases.public:
-                        pen_color = QtCore.Qt.red
-                        pen = QtGui.QPen(pen_color)
-                        pen.setWidth(penradius)
-                        painter.setPen(pen)
-                    else:
-                        icn = geticon('clean')
-
-
-                if icn:
-                    icn.paint(painter, dot_x-5, dot_y-5, 17, 17)
-                elif phase == phases.draft:
-                    painter.drawRect(dot_x, dot_y, radius, radius)
-                elif phase == phases.secret:
-                    P = QtCore.QPointF
-                    painter.drawPolygon(
-                        P(dot_x + (radius//2), dot_y),
-                        P(dot_x, dot_y + radius),
-                        P(dot_x + radius, dot_y+radius)
-                        )
-                else:
-                    painter.drawEllipse(dot_x, dot_y, radius, radius)
-                painter.end()
-                ret = QtCore.QVariant(pix)
-                return ret
-        return nullvariant
-
     def headerData(self, section, orientation, role):
-        if orientation == QtCore.Qt.Horizontal and role == QtCore.Qt.DisplayRole:
-            return QtCore.QVariant(self._columns[section])
-        return nullvariant
+        if orientation == Qt.Horizontal and role == Qt.DisplayRole:
+            return self._columns[section]
+        return None
 
     def maxWidthValueForColumn(self, column):
         column = self._columns[column]
@@ -379,14 +286,13 @@ class HgRepoListModel(QtCore.QAbstractTableModel, HgRepoListWalker):
         self.notify_data_changed()
 
     def notify_data_changed(self):
-        self.emit(SIGNAL("layoutChanged()"))
+        self.layoutChanged.emit()
 
     def indexFromRev(self, rev):
         self.ensureBuilt(rev=rev)
         row = self.rowFromRev(rev)
         if row is not None:
             return self.index(row, 0)
-        return None
 
 class FileRevModel(HgRepoListModel):
     """
@@ -428,7 +334,7 @@ class FileRevModel(HgRepoListModel):
             # works with both versions.
             self.heads = [fl.index[fl.rev(x)][4] for x in fl.heads()]
             self.ensureBuilt(row=self.fill_step/2)
-            QtCore.QTimer.singleShot(0, Curry(self.emit, SIGNAL('filled')))
+            QTimer.singleShot(0, lambda: self.filled.emit())
             self._fill_timer = self.startTimer(500)
         else:
             self.graph = None
@@ -438,7 +344,7 @@ class FileRevModel(HgRepoListModel):
 replus = re.compile(r'^[+][^+].*', re.M)
 reminus = re.compile(r'^[-][^-].*', re.M)
 
-class HgFileListModel(QtCore.QAbstractTableModel):
+class HgFileListModel(QAbstractTableModel):
     """
     Model used for listing (modified) files of a given Hg revision
     """
@@ -450,7 +356,7 @@ class HgFileListModel(QtCore.QAbstractTableModel):
         """
         data is a HgHLRepo instance
         """
-        QtCore.QAbstractTableModel.__init__(self, parent)
+        QAbstractTableModel.__init__(self, parent)
         self.repo = repo
         self._datacache = {}
         self.load_config()
@@ -464,7 +370,7 @@ class HgFileListModel(QtCore.QAbstractTableModel):
     def toggleFullFileList(self):
         self._fulllist = not self._fulllist
         self.loadFiles()
-        self.emit(SIGNAL('layoutChanged()'))
+        self.layoutChanged.emit()
 
     def load_config(self):
         cfg = HgConfig(self.repo.ui)
@@ -481,7 +387,7 @@ class HgFileListModel(QtCore.QAbstractTableModel):
         if w != self.diffwidth:
             self.diffwidth = w
             self._datacache = {}
-            self.emit(SIGNAL('dataChanged(const QModelIndex &, const QModelIndex & )'),
+            self.dataChanged.emit(
                       self.index(1, 0),
                       self.index(1, self.rowCount()))
 
@@ -526,7 +432,7 @@ class HgFileListModel(QtCore.QAbstractTableModel):
         if filename in self._filesdict:
             row = self._files.index(self._filesdict[filename])
             return self.index(row, 0)
-        return QtCore.QModelIndex()
+        return QModelIndex()
 
     def _filterFile(self, filename, ctxfiles):
         if self._fulllist:
@@ -545,6 +451,7 @@ class HgFileListModel(QtCore.QAbstractTableModel):
                 bfile = isbfile(f)
                 if bfile:
                     desc = desc.replace('.hgbfiles'+os.sep, '')
+                desc = tounicode(desc)
                 _files.append({'path': f, 'flag': flag, 'desc': desc, 'bfile': bfile,
                                'parent': parent, 'fromside': fromside,
                                'infiles': f in ctxfiles})
@@ -571,7 +478,7 @@ class HgFileListModel(QtCore.QAbstractTableModel):
             self.current_ctx = ctx
             self._datacache = {}
             self.loadFiles()
-            self.emit(SIGNAL("layoutChanged()"))
+            self.layoutChanged.emit()
 
     def fillFileStats(self):
         """
@@ -589,9 +496,8 @@ class HgFileListModel(QtCore.QAbstractTableModel):
             if nextfill is not None:
                 row, col = nextfill
                 idx = self.index(row, col)
-                self.emit(SIGNAL('dataChanged(const QModelIndex &, const QModelIndex &)'),
-                          idx, idx)
-            QtCore.QTimer.singleShot(10, lambda self=self: self._fill_one_step())
+                self.dataChanged.emit(idx, idx)
+            QTimer.singleShot(10, lambda: self._fill_one_step())
 
         except StopIteration:
             self._fill_iter = None
@@ -606,7 +512,7 @@ class HgFileListModel(QtCore.QAbstractTableModel):
             if desc['flag'] == '=' and self._displaydiff:
                 try:
                     diff = revdiff(self.repo, self.current_ctx, None, files=[filename])
-                    tot = self.current_ctx.filectx(filename).data().count('\n')
+                    tot = tounicode(self.current_ctx.filectx(filename).data()).count('\n')
                     add = len(replus.findall(diff))
                     rem = len(reminus.findall(diff))
                 except (LookupError, TypeError): # unknown revision and mq support
@@ -625,19 +531,19 @@ class HgFileListModel(QtCore.QAbstractTableModel):
                     oldname, node = m
                     if oldname in removed:
                         # removed.remove(oldname) XXX
-                        desc['renamedfrom'] = (oldname, node)
+                        desc['renamedfrom'] = (tounicode(oldname), node)
                         desc['flag'] = '='
-                        desc['desc'] += '\n (was %s)' % oldname
+                        desc['desc'] += u'\n (was %s)' % tounicode(oldname)
                     else:
-                        desc['copiedfrom'] = (oldname, node)
+                        desc['copiedfrom'] = (tounicode(oldname), node)
                         desc['flag'] = '='
-                        desc['desc'] += '\n (copy of %s)' % oldname
+                        desc['desc'] += u'\n (copy of %s)' % tounicode(oldname)
                     yield row, 0
             yield None
 
     def data(self, index, role):
         if not index.isValid() or index.row()>len(self) or not self.current_ctx:
-            return nullvariant
+            return None
         row = index.row()
         column = index.column()
         current_file_desc = self._files[row]
@@ -645,7 +551,7 @@ class HgFileListModel(QtCore.QAbstractTableModel):
         stats = current_file_desc.get('stats')
         if column == 1:
             if stats is not None:
-                if role == QtCore.Qt.DecorationRole:
+                if role == Qt.DecorationRole:
                     tot, add, rem = stats
                     w = self.diffwidth - 20
                     h = 20
@@ -654,36 +560,36 @@ class HgFileListModel(QtCore.QAbstractTableModel):
                     nm = int(w*rem/tot)
                     nd = w-np-nm
 
-                    pix = QtGui.QPixmap(w+10, h)
-                    pix.fill(QtGui.QColor(0,0,0,0))
-                    painter = QtGui.QPainter(pix)
+                    pix = QPixmap(w+10, h)
+                    pix.fill(QColor(0,0,0,0))
+                    painter = QPainter(pix)
 
                     for x0,w0, color in ((0, nm, 'red'),
                                          (nm, np, 'green'),
                                          (nm+np, nd, 'gray')):
-                        color = QtGui.QColor(color)
+                        color = QColor(color)
                         painter.setBrush(color)
                         painter.setPen(color)
                         painter.drawRect(x0+5, 0, w0, h-3)
-                    painter.setBrush(QtGui.QColor(0,0,0,0))
-                    pen = QtGui.QPen(QtCore.Qt.black)
+                    painter.setBrush(QColor(0,0,0,0))
+                    pen = QPen(Qt.black)
                     pen.setWidth(0)
                     painter.setPen(pen)
                     painter.drawRect(5, 0, w+1, h-3)
                     painter.end()
-                    return QtCore.QVariant(pix)
-                elif role == QtCore.Qt.ToolTipRole:
+                    return pix
+                elif role == Qt.ToolTipRole:
                     tot, add, rem = stats
                     msg = "Diff stats:<br>"
                     msg += "&nbsp;<b>File:&nbsp;</b>%s lines<br>" % tot
                     msg += "&nbsp;<b>added lines:&nbsp;</b> %s<br>" % add
                     msg += "&nbsp;<b>removed lines:&nbsp;</b> %s" % rem
-                    return QtCore.QVariant(msg)
+                    return msg
 
         elif column == 0:
-            if role in (QtCore.Qt.DisplayRole, QtCore.Qt.ToolTipRole):
-                return QtCore.QVariant(current_file_desc['desc'])
-            elif role == QtCore.Qt.DecorationRole:
+            if role in (Qt.DisplayRole, Qt.ToolTipRole):
+                return tounicode(current_file_desc['desc'])
+            elif role == Qt.DecorationRole:
                 if self._fulllist and ismerge(self.current_ctx):
                     icn = None
                     if current_file_desc['infiles']:
@@ -693,17 +599,17 @@ class HgFileListModel(QtCore.QAbstractTableModel):
                     elif current_file_desc['fromside'] == 'right':
                         icn = geticon('right')
                     if icn:
-                        return QtCore.QVariant(icn.pixmap(20,20))
-            elif role == QtCore.Qt.FontRole:
+                        return icn.pixmap(20,20)
+            elif role == Qt.FontRole:
                 if self._fulllist and current_file_desc['infiles']:
-                    font = QtGui.QFont()
+                    font = QFont()
                     font.setBold(True)
-                    return QtCore.QVariant(font)
-            elif role == QtCore.Qt.ForegroundRole:
+                    return font
+            elif role == Qt.ForegroundRole:
                 color = self._flagcolor.get(current_file_desc['flag'], 'black')
                 if color is not None:
-                    return QtCore.QVariant(QtGui.QColor(color))
-        return nullvariant
+                    return QColor(color)
+        return None
 
     def headerData(self, section, orientation, role):
         if ismerge(self.current_ctx):
@@ -714,11 +620,9 @@ class HgFileListModel(QtCore.QAbstractTableModel):
         else:
             header = ('File', 'Diff')
 
-        if orientation == QtCore.Qt.Horizontal and role == QtCore.Qt.DisplayRole:
-            return QtCore.QVariant(header[section])
-
-        return nullvariant
-
+        if orientation == Qt.Horizontal and role == Qt.DisplayRole:
+            return header[section]
+        return None
 
 
 class TreeItem(object):
@@ -763,13 +667,13 @@ class TreeItem(object):
             yield ch
 
 
-class ManifestModel(QtCore.QAbstractItemModel):
+class ManifestModel(QAbstractItemModel):
     """
     Qt model to display a hg manifest, ie. the tree of files at a
     given revision. To be used with a QTreeView.
     """
     def __init__(self, repo, rev, parent=None):
-        QtCore.QAbstractItemModel.__init__(self, parent)
+        QAbstractItemModel.__init__(self, parent)
 
         self.repo = repo
         self.changectx = self.repo.changectx(rev)
@@ -777,27 +681,27 @@ class ManifestModel(QtCore.QAbstractItemModel):
 
     def data(self, index, role):
         if not index.isValid():
-            return QtCore.QVariant()
+            return None
 
-        if role != QtCore.Qt.DisplayRole:
-            return QtCore.QVariant()
+        if role != Qt.DisplayRole:
+            return None
 
         item = index.internalPointer()
-        return QtCore.QVariant(item.data(index.column()))
+        return tounicode(item.data(index.column()))
 
     def flags(self, index):
         if not index.isValid():
-            return QtCore.Qt.ItemIsEnabled
-        return QtCore.Qt.ItemIsEnabled | QtCore.Qt.ItemIsSelectable
+            return Qt.ItemIsEnabled
+        return Qt.ItemIsEnabled | Qt.ItemIsSelectable
 
     def headerData(self, section, orientation, role):
-        if orientation == QtCore.Qt.Horizontal and role == QtCore.Qt.DisplayRole:
-            return QtCore.QVariant(self.rootItem.data(section))
-        return QtCore.QVariant()
+        if orientation == Qt.Horizontal and role == Qt.DisplayRole:
+            return self.rootItem.data(section)
+        return None
 
     def index(self, row, column, parent):
         if row < 0 or column < 0 or row >= self.rowCount(parent) or column >= self.columnCount(parent):
-            return QtCore.QModelIndex()
+            return QModelIndex()
 
         if not parent.isValid():
             parentItem = self.rootItem
@@ -807,17 +711,17 @@ class ManifestModel(QtCore.QAbstractItemModel):
         if childItem is not None:
             return self.createIndex(row, column, childItem)
         else:
-            return QtCore.QModelIndex()
+            return QModelIndex()
 
     def parent(self, index):
         if not index.isValid():
-            return QtCore.QModelIndex()
+            return QModelIndex()
 
         childItem = index.internalPointer()
         parentItem = childItem.parent()
 
         if parentItem == self.rootItem:
-            return QtCore.QModelIndex()
+            return QModelIndex()
 
         return self.createIndex(parentItem.row(), 0, parentItem)
 
